# frozen_string_literal: true

require 'json'
require 'faraday'

module Dadatas
  # Call API DADATA
  class ApiCall
    attr_reader :url, :config, :params

    def initialize(url, config, params)
      @url = url
      @config = config
      @params = params
    end

    def call
      response = perform_request(url, params)

      return response_handler(response) if response.status.eql?(200)

      error_response(response, response.body)
    end

    def response_handler(response)
      response_body = JSON.parse(response.body, symbolize_names: true)

      return success(response_body) if response_body.is_a?(Array)

      return success(response_body[:suggestions]) if response_body.key?(:suggestions)

      error_response(response, response_body)
    end

    def error_response(response, response_body)
      [false, { code: response.status, message: response_body }]
    end

    def success(response)
      new_response = response.map do |result|
        result[:data].merge!(req_value: result[:value]) if result.key?(:data)
        result
      end

      [response.any?, new_response]
    end

    def header
      {
        'Content-Type' => 'application/json',
        'Accept' => 'application/json',
        'Authorization' => "Token #{@config[:api_key]}",
        'X-Secret' => @config[:secret_key].to_s
      }
    end

    def perform_request(url, params)
      Faraday.post(url, params.to_json, header)
    rescue StandardError => e
      e
    end
  end
end
